/*
 * Created on 27.09.2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */


package gnu.java.beans.decoder;


/** AbstractCreatableObjectContext is the base class for all Context implementation
 * which create a result object in their lifetime. The provides means for preventing
 * to create the object twice.
 *
 * @author Robert Schuster
 *
 */
abstract class AbstractCreatableObjectContext extends AbstractObjectContext
{
  AbstractCreatableObjectContext()
  {
  }

  /** Adds a parameter object to this Context if the result object has not been
   * created yet. Otherwise an AssemblerException is thrown.
   */
  public final void addParameterObject(Object o) throws AssemblyException
  {
    if (object == null)
      addParameterObjectImpl(o);
    else
      throw new AssemblyException(new IllegalStateException("No more objects are allowed when the object as already created."));
  }

  /** Adds a parameter object to this Context.
   *
   * @param o
   */
  protected abstract void addParameterObjectImpl(Object o);

  /** Creates the result object if it does not exist already.
   */
  public final void notifyStatement(Context outerContext)
    throws AssemblyException
  {
    if (object != null)
      return;

    object = createObject(outerContext);
  }

  /** Creates the result object. This method is called only once.
   *
   * @param outerContext
   * @return
   * @throws AssemblerException
   */
  protected abstract Object createObject(Context outerContext)
    throws AssemblyException;

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#endContext(gnu.java.beans.decoder.Context)
   */
  public final Object endContext(Context outerContext)
    throws AssemblyException
  {
    notifyStatement(outerContext);
    return object;
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#subContextFailed()
   */
  public boolean subContextFailed()
  {
    // returns true when the AbstractObjectContext not created the object yet because only then a failed
    // subcontext automatically let this context fail
    return object == null;
  }
}
