/*
 * Created on 27.09.2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */


package gnu.java.beans.decoder;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/** AbstractObjectContext is the base for all Context implementations which
 * create or provide a result object during their lifetime.
 *
 * This class provides the implementation for an indexed get and set method.
 * But this does not mean that the result object supports these operation.
 *
 * @author Robert Schuster
 *
 */
abstract class AbstractObjectContext extends AbstractContext
{
  protected Object object;

  AbstractObjectContext()
  {
  }

  /** Sets the result object of the Context.
   *
   * @param o
   */
  protected final void setObject(Object o)
  {
    object = o;
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#set(int, java.lang.Object)
   */
  public final void set(int index, Object o) throws AssemblyException
  {
    try
      {
	Method method = object.getClass().getMethod("set",
	                                            new Class[]
	                                            {
	                                              Integer.TYPE,
	                                              Object.class
	                                            });

	method.invoke(object, new Object[] { new Integer(index), o });
      }
    catch (NoSuchMethodException nsme)
      {
	throw new AssemblyException(nsme);
      }
    catch (InvocationTargetException ite)
      {
	throw new AssemblyException(ite.getCause());
      }
    catch (IllegalAccessException iae)
      {
	throw new AssemblyException(iae);
      }
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#get(int)
   */
  public final Object get(int index) throws AssemblyException
  {
    try
      {
	Method method = object.getClass().getMethod("get",
	                                            new Class[] { Integer.TYPE });

	return method.invoke(object, new Object[] { new Integer(index) });
      }
    catch (NoSuchMethodException nsme)
      {
	throw new AssemblyException(nsme);
      }
    catch (InvocationTargetException ite)
      {
	throw new AssemblyException(ite.getCause());
      }
    catch (IllegalAccessException iae)
      {
	throw new AssemblyException(iae);
      }
  }

  public final Object getResult()
  {
    return object;
  }
}
