/*
 * Created on 24.09.2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */


package gnu.java.beans.decoder;

import java.beans.ExceptionListener;
import java.util.HashMap;

import org.xml.sax.Attributes;

/** ArrayHandler processes the <array> tag. Depending on the existance of the 'length' attribute a Context for
 * a fixed-size or growable array is created.
 *
 * @author Robert Schuster
 */
class ArrayHandler extends AbstractElementHandler
{
  /** Contains a mapping between a textual description of a primitive type (like "byte") and
   * its corresponding wrapper class. This allows it to easily construct Array objects for
   * primitive data types.
   */
  private static HashMap typeMap = new HashMap();

  static
    {
      typeMap.put("byte", Byte.TYPE);
      typeMap.put("short", Short.TYPE);
      typeMap.put("int", Integer.TYPE);
      typeMap.put("long", Long.TYPE);

      typeMap.put("float", Float.TYPE);
      typeMap.put("double", Double.TYPE);

      typeMap.put("boolean", Boolean.TYPE);

      typeMap.put("char", Character.TYPE);
    }

  /**
   * @param PersistenceParser
   */
  ArrayHandler(ElementHandler parent)
  {
    super(parent, true);
  }

  protected Context startElement(Attributes attributes, ExceptionListener exceptionListener)
    throws AssemblyException, AssemblyException
  {
    String id = attributes.getValue("id");
    String className = attributes.getValue("class");

    if (className != null)
      {
	try
	  {
	    Class klass;

	    if (typeMap.containsKey(className))
	      klass = (Class) typeMap.get(className);
	    else
	      klass = instantiateClass(className);

	    String length = attributes.getValue("length");
	    if (length != null)
	      // creates Array with predefined length
	      return new ArrayContext(id, klass, Integer.parseInt(length));
	    else
	      // creates Array without length restriction
	      return new GrowableArrayContext(id, klass);
	  }
	catch (ClassNotFoundException cnfe)
	  {
	    throw new AssemblyException(cnfe);
	  }
	catch (NumberFormatException nfe)
	  {
	    throw new AssemblyException(nfe);
	  }
      }

    throw new AssemblyException(new IllegalArgumentException("Missing 'class' attribute in <array> tag."));
  }
}
