/*
 * Created on 27.09.2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */


package gnu.java.beans.decoder;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

/** A ConstructorContext is a Context implementation which collects the parameters for a constructor
 * call and instantiates the result object using that constructor. After that sub-Contexts can invoke
 * methods on the result object.
 *
 * The constructor is invoked when a sub-Context is a statement or the Context ends.
 *
 * @author Robert Schuster
 */
class ConstructorContext extends AbstractCreatableObjectContext
{
  private ArrayList arguments = new ArrayList();
  private Class klass;

  ConstructorContext(String id, Class newClass)
  {
    setId(id);
    // sets superclass field 
    klass = newClass;
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#addObject(java.lang.Object)
   */
  protected void addParameterObjectImpl(Object o)
  {
    arguments.add(o);
  }

  protected Object createObject(Context outerContext)
    throws AssemblyException
  {
    Object[] args = arguments.toArray();

    try
      {
	Constructor constructor = MethodFinder.getConstructor(klass, args);

	// instantiates object (klass field gets re-set by superclass)
	return constructor.newInstance(args);
      }
    catch (NoSuchMethodException nsme)
      {
	throw new AssemblyException(nsme);
      }
    catch (InvocationTargetException ite)
      {
	throw new AssemblyException(ite.getCause());
      }
    catch (IllegalAccessException iae)
      {
	throw new AssemblyException(iae);
      }
    catch (InstantiationException ie)
      {
	throw new AssemblyException(ie);
      }
  }
  
}
