/*
 * Created on 05.10.2004
 *
 */


package gnu.java.beans.decoder;

import java.beans.ExceptionListener;

import org.xml.sax.Attributes;

/** ElementHandler manages a Context instance and interacts with
 * its parent and child handlers.
 *
 * @author Robert Schuster
 */
interface ElementHandler
{
  /** Evaluates the attributes and creates a Context instance.
   * If the creation of the Context instance fails the ElementHandler
   * is marked as failed which may affect the parent handler other.
   *
   * @param attributes Attributes of the XML tag.
   */
  void start(Attributes attributes, ExceptionListener exceptionListener);

  /** Post-processes the Context.
   */
  void end(ExceptionListener exceptionListener);

  /** Adds characters from the body of the XML tag to the buffer.
   *
   * @param ch
   * @param start
   * @param length
   * @throws SAXException
   */
  void characters(char[] ch, int start, int length);

  /** Returns whether a subelement of the given name is allowed. The rules
   * for evaluating this are derived from the javabeans.dtd which can be found
   * here: http://java.sun.com/products/jfc/tsc/articles/persistence3
   * 
   * @param subElementName 
   * @return
   */
  boolean isSubelementAllowed(String subElementName);

  /** Provides the same functionality as Class.forName() but allows the decoder
   * to use a different class loader.
   * 
   * @param className
   * @return
   * @throws ClassNotFoundException
   */ 
  Class instantiateClass(String className) throws ClassNotFoundException;

  /** Notifies the handler's Context that its child Context will not return
   * a value back. Some Context variants need this information to know when
   * a method or a constructor call can be made.
   *
   * This method is called by a child handler.
   */
  void notifyStatement(ExceptionListener exceptionListener);

  /** Returns whether this handler has failed.
   *
   * This is used to skip child elements.
   *
   * @return Whether this handler has failed.
   */
  boolean hasFailed();

  /** Returns the Context instance this handler is working on.
   * 
   * @return The handler's Context instance.
   */
  Context getContext();

  /** Notifies the handler that its Context failed and starts a recursive
   * invocation of the parent handler if it is affected by that failure.
   * 
   * Although the method is a public API member it is only used internally.
   */
  void notifyContextFailed();

  /** Stores the object under the given id. The object is not stored if the
   * id is null.
   * 
   * @param objectId
   * @param o
   */
  void putObject(String objectId, Object o);

  Object getObject(String objectId) throws AssemblyException;

  ElementHandler getParent();
}
