/*
 * Created on 27.09.2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */


package gnu.java.beans.decoder;

import java.lang.reflect.Array;
import java.util.ArrayList;

/** A Context implementation for a growable array. The array
 * elements have to be set using expressions.
 *
 * XXX: add support for statements
 *
 * @author Robert Schuster
 */
class GrowableArrayContext extends AbstractContext
{
  private Class klass;
  private ArrayList list = new ArrayList();
  private Object array;

  GrowableArrayContext(String id, Class newClass)
  {
    setId(id);
    klass = newClass;
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#addObject(java.lang.Object)
   */
  public void addParameterObject(Object o) throws AssemblyException
  {
    if (! klass.isInstance(o))
      throw new AssemblyException(new IllegalArgumentException("Cannot add object "
                                                                + o
                                                                + " to array where the elements are of class "
                                                                + klass));

    list.add(o);
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#reportStatement()
   */
  public void notifyStatement(Context outerContext) throws AssemblyException
  {
    throw new AssemblyException(new IllegalArgumentException("Statements inside a growable array are not allowed."));
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#endContext(gnu.java.beans.decoder.Context)
   */
  public Object endContext(Context outerContext) throws AssemblyException
  {
    if (array == null)
      {
	array = Array.newInstance(klass, list.size());

	for (int i = 0; i < list.size(); i++)
	  Array.set(array, i, list.get(i));
      }

    return array;
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#subContextFailed()
   */
  public boolean subContextFailed()
  {
    // returns false to indicate that assembling the array does not fail only because
    // a subelement failed
    return false;
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#set(int, java.lang.Object)
   */
  public void set(int index, Object o) throws AssemblyException
  {
    if (array == null)
      {
	if (klass.isInstance(o))
	  list.add(index, o);
	else
	  throw new AssemblyException(new IllegalArgumentException("Argument is not compatible to array component type."));
      }
    else
      Array.set(array, index, o);
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#get(int)
   */
  public Object get(int index) throws AssemblyException
  {
    if (array == null)
      return list.get(index);
    else
      return Array.get(array, index);
  }

  public Object getResult()
  {
    return array;
  }
}
