/*
 * Created on 27.09.2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */


package gnu.java.beans.decoder;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/** PropertyContext is a Context implementation that is very similar to MethodContext
 * and IndexContext. The sole purpose of PropertyContext to find out whether it should
 * 'set' or 'get' a certain property. This decision is made using the number of
 * arguments.
 * When the method call has to be made and there is no argument we 'get' the property.
 * With one argument it is 'set'.
 *
 * @author Robert Schuster
 *
 */
class PropertyContext extends AbstractObjectContext
{
  private Object argument;
  private String propertyName;
  private String prefix = "get";
  private boolean methodCalled;

  PropertyContext(String id, String newPropertyName)
  {
    setId(id);
    propertyName = newPropertyName;
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#addObject(java.lang.Object)
   */
  public void addParameterObject(Object o) throws AssemblyException
  {
    if (methodCalled)
      throw new AssemblyException(new IllegalArgumentException("Cannot add parameter object when method was already called."));

    if (argument != null)
      throw new AssemblyException(new IllegalArgumentException("Property attribut allows zero or one argument only."));

    argument = o;
	setStatement(true);
    prefix = "set";
  }

  /* (non-Javadoc)
   * @see gnu.java.beans.decoder.Context#endContext(gnu.java.beans.decoder.Context)
   */
  public void notifyStatement(Context outerContext) throws AssemblyException
  {
    if (methodCalled)
      return;
    methodCalled = true;

    Object outerObject = outerContext.getResult();

    if (outerObject == null)
      throw new AssemblyException(new NullPointerException("No object to access property "
	  + propertyName));
    

    // converts property name into a method name
    String methodName = prefix + propertyName.substring(0, 1).toUpperCase()
                        + propertyName.substring(1);

    // prepares the argument
    Object[] args = (argument != null) ? new Object[] { argument } : null;

    try
      {
	Method method = MethodFinder.getMethod(outerObject.getClass(),
	                                       methodName, args);

	// stores the result whether it is available or not
	setObject(method.invoke(outerObject, args));
      }
    catch (NoSuchMethodException nsme)
      {
	throw new AssemblyException(nsme);
      }
    catch (InvocationTargetException ite)
      {
	throw new AssemblyException(ite.getCause());
      }
    catch (IllegalAccessException iae)
      {
	throw new AssemblyException(iae);
      }
  }

  public Object endContext(Context outerContext) throws AssemblyException
  {
    notifyStatement(outerContext);

    return getResult();
  }

  public boolean subContextFailed()
  {
    return ! methodCalled;
  }
}
