/*
 * Created on 05.10.2004
 *
 */


package gnu.java.beans.decoder;

import java.beans.ExceptionListener;

import org.xml.sax.Attributes;

/** XML element handler that is specialized on tags that contain a simple string in their
 * body which has to be parsed in a specific way.
 * All of these tags have in common that they do not accept attributes. A warning is
 * send to the parser's ExceptionListener when one or more attributes exist.
 *
 * @author Robert Schuster
 */
abstract class SimpleHandler extends AbstractElementHandler
{
  private ObjectContext context;

  /**
   * @param PersistenceParser
   */
  SimpleHandler(ElementHandler parent)
  {
    super(parent, false);

    // SimpleHandler do not accept any subelements
  }

  protected final Context startElement(Attributes attributes, ExceptionListener exceptionListener)
    throws AssemblyException
  {
  	
    // note: simple elements should not have any attributes. We inform
    // the user of this syntactical but uncritical problem by sending
    // an IllegalArgumentException for each unneccessary attribute
    int size = attributes.getLength();
    for (int i = 0; i < size; i++) {
            String attributeName = attributes.getQName(i);
            Exception e =
                    new IllegalArgumentException(
                            "Unneccessary attribute '"
                                    + attributeName
                                    + "' discarded.");
            exceptionListener.exceptionThrown(e);
    }
    
    return context = new ObjectContext();
  }

  public void endElement(String characters)
    throws AssemblyException, AssemblyException
  {
    // reports the number when the character data can be parsed
    try
      {
	context.setObject(parse(characters));
      }
    catch (NumberFormatException nfe)
      {
	throw new AssemblyException(nfe);
      }
  }

  /** Returns an object that is created from the given characters. If the string is
   * converted into a number a NumberFormatException is cathed and reported
   * appropriately.
   *
   * @param characters A string of characters that has to be processed in some way.
   * @return An Object instance generated from the given data.
   * @throws AssemblerException When the string was invalid.
   * @throws NumberFormatException When the string could not be parsed into a number.
   */
  protected abstract Object parse(String characters)
    throws AssemblyException, NumberFormatException;
}
