/*
 * Created on 24.09.2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */


package gnu.java.beans.decoder;

import java.beans.ExceptionListener;

import org.xml.sax.Attributes;

public class VoidHandler extends AbstractElementHandler
{
  /**
   * @param PersistenceParser
   */
  VoidHandler(ElementHandler parent)
  {
    super(parent, true);
  }

  protected Context startElement(Attributes attributes, ExceptionListener exceptionListener)
    throws AssemblyException
  {
    Context ctx = startElementImpl(attributes);
    ctx.setStatement(true);

    return ctx;
  }

  private Context startElementImpl(Attributes attributes)
    throws AssemblyException
  {
    String id = attributes.getValue("id");
    String className = attributes.getValue("class");
    String methodName = attributes.getValue("method");
    String propertyName = attributes.getValue("property");
    String index = attributes.getValue("index");

    if (className != null)
      {
	try
	  {
	    Class klass = instantiateClass(className);

	    // class name exists which means that we are in a static context.
	    // so we may want to ...
	    // run a constructor if methodName is "new" or null
	    if (methodName == null || methodName.equals("new"))
	      // if the id is null the result cannot be by the decoder accessed but the
	      // constructor may have side effects (e.g. registering itself in a global registry)
	      return new ConstructorContext(id, klass);

	    // (falling through is important!)
	    // run a static method on the given class (if methodName exists, which is implied already) 
	    return new StaticMethodContext(id, klass, methodName);
	  }
	catch (ClassNotFoundException cnfe)
	  {
	    throw new AssemblyException(cnfe);
	  }
      }
    else
      {
	// className does not exist which means we are in the context of
	// some object and want to ...
	// access an element by index
	if (index != null)
	  {
	    // note: whether this resolves into get(i) or set(i, o) depends on the
	    // number of arguments and is decided by the ObjectAssembler
	    try
	      {
		return new IndexContext(id, Integer.parseInt(index));
	      }
	    catch (NumberFormatException nfe)
	      {
		throw new AssemblyException(nfe);
	      }
	  }

	// access a method if methodName exists
	if (methodName != null)
	  return new MethodContext(id, methodName);

	// (falling through is important!)
	// access a property if a propertyName exists
	if (propertyName != null && propertyName.length() > 0)
	  // this is reported as an ordinary method invocation where the propertyName is
	  // converted into a 'setter'-method name: convert first character of property name
	  // to upper case and prepend 'set'
	  // Note: This will be a setter-method because the <void> tag implies that no return
	  // value is expected (but a side effect)
	  return new PropertyContext(id, propertyName);
      }

    // if code reaches this point the tag has wrong attributes. The following test
    // does not make it better but can provide are more specific error message for
    // a common mistake: <void> tags are not allowed to have an idref attribute
    throw new AssemblyException(new IllegalArgumentException((attributes
                                                               .getValue("idref") == null)
                                                              ? "Missing attributes for <void> tag"
                                                              : "<void> does not support 'idref' attribute."));
  }
}
